@@ -4,20 +4,27 @@ import sys
 from ssort._dependencies import (
     class_statements_initialisation_graph,
     class_statements_runtime_graph,
-    statements_graph,
+    module_statements_graph,
+)
+from ssort._exceptions import (
+    DecodingError,
+    ParseError,
+    ResolutionError,
+    UnknownEncodingError,
+    WildcardImportError,
 )
 from ssort._graphs import (
     is_topologically_sorted,
     replace_cycles,
     topological_sort,
 )
-from ssort._parsing import split, split_class
+from ssort._parsing import parse, split_class
 from ssort._statements import (
     statement_bindings,
     statement_node,
     statement_text,
 )
-from ssort._utils import sort_key_from_iter
+from ssort._utils import detect_encoding, sort_key_from_iter
 
 SPECIAL_PROPERTIES = [
     "__doc__",
@@ -201,10 +208,14 @@ def _is_regular_operation(statement):
 
 def _is_property(statement):
     node = statement_node(statement)
-    if not isinstance(node, (ast.Assign, ast.AnnAssign, ast.AugAssign)):
-        return False
 
-    return True
+    return isinstance(node, (ast.Assign, ast.AnnAssign, ast.AugAssign))
+
+
+def _is_class(statement):
+    node = statement_node(statement)
+
+    return isinstance(node, ast.ClassDef)
 
 
 def _statement_binding_sort_key(binding_key):
@@ -246,6 +257,8 @@ def _statement_text_sorted_class(statement):
         statements, _is_regular_operation
     )
 
+    inner_classes, statements = _partition(statements, _is_class)
+
     properties, statements = _partition(statements, _is_property)
 
     methods, statements = statements, []
@@ -263,6 +276,9 @@ def _statement_text_sorted_class(statement):
         ),
     )
 
+    # Inner classes (in original order).
+    sorted_statements += inner_classes
+
     # Regular properties (in original order).
     sorted_statements += properties
 
@@ -292,9 +308,12 @@ def _statement_text_sorted_class(statement):
         sorted_statements, graph=initialisation_graph
     )
 
-    # Attempt to resolve soft dependencies, but with hard dependencies taking
-    # priority, and keeping existing order where there are cycles.
-    runtime_graph = class_statements_runtime_graph(sorted_statements)
+    # Attempt to resolve soft dependencies on private attributes, but with hard
+    # dependencies taking priority, and always preserving the original order
+    # where there are cycles.
+    runtime_graph = class_statements_runtime_graph(
+        sorted_statements, ignore_public=True
+    )
     runtime_graph.update(initialisation_graph)
     replace_cycles(runtime_graph, key=sort_key_from_iter(sorted_statements))
 
@@ -319,10 +338,149 @@ def statement_text_sorted(statement):
     return statement_text(statement)
 
 
-def ssort(text, *, filename="<unknown>"):
-    statements = list(split(text, filename=filename))
+def _on_unknown_encoding_ignore(message, **kwargs):
+    pass
 
-    graph = statements_graph(statements)
+
+def _on_unknown_encoding_raise(message, *, encoding, **kwargs):
+    raise UnknownEncodingError(message, encoding=encoding)
+
+
+def _interpret_on_unknown_encoding_action(on_unknown_encoding):
+    if on_unknown_encoding == "ignore":
+        return _on_unknown_encoding_ignore
+
+    if on_unknown_encoding == "raise":
+        return _on_unknown_encoding_raise
+
+    return on_unknown_encoding
+
+
+def _on_decoding_error_ignore(message, **kwargs):
+    pass
+
+
+def _on_decoding_error_raise(message, **kwargs):
+    raise DecodingError(message)
+
+
+def _interpret_on_decoding_error_action(on_decoding_error):
+    if on_decoding_error == "ignore":
+        return _on_decoding_error_ignore
+
+    if on_decoding_error == "raise":
+        return _on_decoding_error_raise
+
+    return on_decoding_error
+
+
+def _on_parse_error_ignore(message, **kwargs):
+    pass
+
+
+def _on_parse_error_raise(message, *, lineno, col_offset, **kwargs):
+    raise ParseError(message, lineno=lineno, col_offset=col_offset)
+
+
+def _interpret_on_parse_error_action(on_parse_error):
+    if on_parse_error == "ignore":
+        return _on_parse_error_ignore
+
+    if on_parse_error == "raise":
+        return _on_parse_error_raise
+
+    return on_parse_error
+
+
+def _on_unresolved_ignore(message, *, name, lineno, col_offset, **kwargs):
+    pass
+
+
+def _on_unresolved_raise(message, *, name, lineno, col_offset, **kwargs):
+    raise ResolutionError(
+        message,
+        name=name,
+        lineno=lineno,
+        col_offset=col_offset,
+    )
+
+
+def _interpret_on_unresolved_action(on_unresolved):
+    if on_unresolved == "ignore":
+        return _on_unresolved_ignore
+
+    if on_unresolved == "raise":
+        return _on_unresolved_raise
+
+    return on_unresolved
+
+
+def _on_wildcard_import_ignore(message, **kwargs):
+    pass
+
+
+def _on_wildcard_import_raise(message, *, lineno, col_offset, **kwargs):
+    raise WildcardImportError(
+        "can't reliably determine dependencies on * import"
+    )
+
+
+def _interpret_on_wildcard_import_action(on_wildcard_import):
+    if on_wildcard_import == "ignore":
+        return _on_wildcard_import_ignore
+
+    if on_wildcard_import == "raise":
+        return _on_wildcard_import_raise
+
+    return on_wildcard_import
+
+
+def ssort(
+    text,
+    *,
+    filename="<unknown>",
+    on_unknown_encoding_error="raise",
+    on_decoding_error="raise",
+    on_parse_error="raise",
+    on_unresolved="raise",
+    on_wildcard_import="raise",
+):
+    on_unknown_encoding_error = _interpret_on_unknown_encoding_action(
+        on_unknown_encoding_error
+    )
+    on_decoding_error = _interpret_on_decoding_error_action(on_decoding_error)
+    on_parse_error = _interpret_on_parse_error_action(on_parse_error)
+    on_unresolved = _interpret_on_unresolved_action(on_unresolved)
+    on_wildcard_import = _interpret_on_wildcard_import_action(
+        on_wildcard_import
+    )
+
+    try:
+        encoding = None
+        if isinstance(text, bytes):
+            encoding = detect_encoding(text)
+            text = text.decode(encoding)
+    except UnknownEncodingError as exc:
+        on_unknown_encoding_error(str(exc), encoding=exc.encoding)
+        return text
+
+    except UnicodeDecodeError as exc:
+        on_decoding_error(str(exc))
+        return text
+
+    try:
+        statements = list(parse(text, filename=filename))
+    except ParseError as exc:
+        on_parse_error(str(exc), lineno=exc.lineno, col_offset=exc.col_offset)
+        return text
+
+    graph = module_statements_graph(
+        statements,
+        on_unresolved=on_unresolved,
+        on_wildcard_import=on_wildcard_import,
+    )
+    if graph is None:
+        return text
 
     replace_cycles(graph, key=sort_key_from_iter(statements))
 
@@ -335,4 +493,7 @@ def ssort(text, *, filename="<unknown>"):
     )
     if output:
         output += "\n"
+
+    if encoding is not None:
+        output = output.encode(encoding)
     return output
